/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * Project		MCAPI+MDMA example - Dual Sharc Talkthrough
 * Description	The purpose of this project is to demonstrate the MCAPI and MDMA
 * 				Inter-Processor Communications (IPC) methods. The project takes
 * 				four channels of audio from the ADAU1979 CODEC into
 * 				Sharc 0 (core 1) L1 SRAM and sends the data directly to
 * 				Sharc 1 (core 2) L1 SRAM using MDMA during runtime. MCAPI is used
 * 				to initialize and synchronize the MDMA channel during initialization.
 * 				The ARM is used during initialization but is not used during runtime.
 *
 * File
 * Description	The two Sharcs come out of reset at approximately the same time.
 * 				This file contains main() for Sharc 2. Sharc 2 manages the DAC.
 * 				The DAC is enabled when the first frame is received from Sharc 1.
 * 				This synchronizes the ADC and the DAC. Audio frames coming from
 * 				Sharc 1 cause an interrupt. The interrupt handler runs the output
 * 				filter. The result is copied into a DAC ping pong buffer.
 *
 */
#include <sys/platform.h>
#include <sys/adi_core.h>
#include <stdio.h>
#include "adi_initialize.h"
#include <drivers/dac/adau1962a/adi_adau1962a.h>
#include "MultiCoreTalkThruSharcSharc_Core2.h"
#include "SHARC_linkInterface.h"


#define TIMESTAMP	if(CycleCountBufferIndex < 1024) {CycleCountBuffer[CycleCountBufferIndex][0] =__LINE__; CycleCountBuffer[CycleCountBufferIndex++][1] = __builtin_emuclk();}

//*****************************************************************************
// External Functions
//*****************************************************************************
extern uint32_t Adau1962aSubmitBuffers(void);
extern uint32_t Adau1962aInit(void);
extern uint32_t Adau1962DoneWithBuffer( void *pBuffer );
extern uint32_t Adau1962Enable( void );


//*****************************************************************************
// Local Variables
//*****************************************************************************
/* Counter to keep track of number of DAC buffers processed */
volatile uint32_t DacCount = 0u;
volatile uint32_t RXBuffersReceived = 0u;
volatile uint8_t  DacStarted = 0u;
volatile void *pSportOutputBuffer = 0;

// Fix for stutter bug in MDMA driver
volatile unsigned long long PreviousCycleTime = 0u;

// These variables are used for instrumenting the code
volatile int CycleCountBufferIndex = 0;
volatile unsigned long CycleCountBuffer[4096][2];
int8_t BufferFromCore1[AUDIO_BUFFER_SIZE];

//*************************************************************************
// Interrupt handler for MDMA transfer complete on slave Sharc
//*************************************************************************
static void DataTransferFromMasterComplete(uint32_t SID, void *pCBParam)
{
	// Fix for stutter
	if((__builtin_emuclk() - PreviousCycleTime) > 1000 )
	{
		PreviousCycleTime = __builtin_emuclk();
		TIMESTAMP
		++RXBuffersReceived;
		if( pSportOutputBuffer != 0 )
			OutputSeriesFilter( &BufferFromCore1[0], (int8_t *)pSportOutputBuffer, AUDIO_BUFFER_SIZE );
	}
}

/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

int main(int argc, char *argv[])
{
	uint32_t 	Result = 0u;
	uint32_t	DMASlaveDestinationAddress;


	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();
	
	/* Begin adding your custom code here */
	/* Initialise ADAU1962A */
	if(Adau1962aInit() != 0u)
	{
		fprintf(stdout, "Core2: Adau1962aInit() failed\n" );
		while(1){asm(" NOP; ");}
	}

	/* Submit DAC buffers */
	if(Adau1962aSubmitBuffers() != 0u)
	{
		fprintf(stdout, "Core2: Adau1962aSubmitBuffers() failed\n" );
		while(1){asm(" NOP; ");}
	}

	// Initialize SHARClink
	// The code will block here until the other core executes.
	DMASlaveDestinationAddress = (uint32_t)&BufferFromCore1[0];
	DMASlaveDestinationAddress &= 0x0000FFFF;
	DMASlaveDestinationAddress |= 0x28A40000; // This assumes L1 block 0
	fprintf( stdout, "Core2: DMA Slave Destination Address = 0x%08X\n", DMASlaveDestinationAddress );
	Result = SHARC_linkSlaveInit( DataTransferFromMasterComplete, (void *)DMASlaveDestinationAddress );
	if(Result != 0u)
	{
		fprintf(stdout, "Core2: SHARClinkSlaveInit() failed\n" );
		while(1){asm(" NOP; ");}
	}

//	fprintf(stdout, "Core2: Entering superloop\n" );

	while(1)
	{
		if( DacStarted == 0 && RXBuffersReceived > 0 )
		{
			// Enable audio output
			Adau1962Enable();
			DacStarted = 1;
//			fprintf(stdout, "Core2: DAC Started\n" ); // EMG - 071116 - Removed to avoid sync issues
		}
	}

	return 0;
}


/*
 * DAC Callback.
 *
 * Parameters
 *  None
 *
 * Returns
 *  None
 *
 */
void DacCallback(void *pCBParam, uint32_t nEvent, void *pArg)
{

    switch(nEvent)
    {
        case ADI_SPORT_EVENT_TX_BUFFER_PROCESSED:
        	/* Update callback count */
        	DacCount++;
        	TIMESTAMP
        	// We can still copy to the buffer after it is submitted to the driver
        	pSportOutputBuffer = pArg;
    		Adau1962DoneWithBuffer( pArg );
        	break;
        default:
            break;
    }
}


