/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * Project		MCAPI+MDMA example - Dual Sharc Talkthrough
 * Description	The purpose of this project is to demonstrate the MCAPI and MDMA
 * 				Inter-Processor Communications (IPC) methods. The project takes
 * 				four channels of audio from the ADAU1979 CODEC into
 * 				Sharc 0 (core 1) L1 SRAM and sends the data directly to
 * 				Sharc 1 (core 2) L1 SRAM using MDMA during runtime. MCAPI is used
 * 				to initialize and synchronize the MDMA channel during initialization.
 * 				The ARM is used during initialization but is not used during runtime.
 *
 * File
 * Description	In order to use MDMA to transfer data between cores, the driver needs
 * 				to be setup in a particular sequence. Sharc 1 is the source or master
 * 				core. It opens the MDMA channel on a specific stream, then uses MCAPI
 * 				to send the Sid to Sharc 2. Sharc 2 is the destination or slave core.
 * 				After receiving the Sid, the slave core installs an interrupt handler
 * 				to handle the MDMA completion event. Audio frames cannot be sent to the
 * 				slave core until the slave sends a message to the master core saying the
 * 				interrupt handler is installed. After the slave core installs its interrupt
 * 				handler, it sends the destination buffer address to the master core. After
 * 				the master core receives the destination buffer address, it starts sending
 * 				audio frames to the destination buffer address.
 *
 */
#include <services/dma/adi_dma.h>
#include <services/mcapi/mcapi.h>
#include "SHARC_linkInterface.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//*****************************************************************************
// Called from Slave Sharc (receiving Sharc) to initialize the SHARC link.
// Uses MCAPI to establish SHARC link. SHARClink does not use MCAPI during runtime.
//
// 1) Master opens MDMA channel
// 2) Master configures MDMA channel
// 3) Master opens MCAPI connection to slave
// 4) Master sends Sid to slave
// 5) Slave waits for Sid from Master
// 6) Slave inserts interrupt handler based on Sid
// 7) Slave sends DMA destination buffer address to Master
// 8) Master waits for buffer address from slave
//
// Installs an interrupt handler at the Sid sent from the master.
//*****************************************************************************
int SHARC_linkSlaveInit( ADI_INT_HANDLER_PTR pfHandler, void *DMASlaveDestinationAddress )
{
	mcapi_endpoint_t 	slave_ep;
	mcapi_endpoint_t 	master_ep;
	mcapi_status_t 		mcapi_status = MCAPI_ERR_GENERAL;
	size_t 				rcv_size;
	uint32_t			nSid;
	size_t				RcvBytes;
	uint32_t			DestAddress;

#ifdef MCAPI_VERBOSE
	char status_buff[MCAPI_MAX_STATUS_MSG_LEN];				// Buffer for MCAPI verbose error reporting
#endif

	//*************************************************************************
	// Open MCAPI connection to master
	//*************************************************************************
	DEBUGMSG(stdout, "Core2: Creating MCAPI slave endpoint\n" );
	slave_ep = mcapi_endpoint_create(1, &mcapi_status);		// Create MCAPI slave endpoint
	if (MCAPI_SUCCESS != mcapi_status)
	{
#ifdef MCAPI_VERBOSE
		mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
	    DEBUGMSG(stdout, "[ERR] mcapi_endpoint_create %s %s %d\n", status_buff, __FILE__, __LINE__);
#endif
	    return(SHARC_LINK_MCAPI_ERROR);
	}
	DEBUGMSG(stdout, "Core2: MCAPI slave endpoint created\n" );

	//*************************************************************************
	// Wait for Sid from master
	//*************************************************************************
	mcapi_msg_recv(slave_ep, &nSid, sizeof(nSid), &RcvBytes, &mcapi_status);
	if (MCAPI_SUCCESS != mcapi_status)						// Block until we receive the Sid from master
	{
#ifdef MCAPI_VERBOSE
		mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
	    DEBUGMSG(stdout, "[ERR] mcapi_msg_recv_i %s %s %d\n", status_buff, __FILE__, __LINE__);
#endif
	    return(SHARC_LINK_MCAPI_ERROR);
	}
	DEBUGMSG( stdout, "Core2: Received 0x%08X Sid from master\n", nSid );

	//*************************************************************************
	// insert interrupt handler based on Sid
	//*************************************************************************
	adi_int_InstallHandler (nSid, pfHandler, NULL, true);
	DEBUGMSG( stdout, "Core2: MDMA interrupt handler installed\n" );

	//*************************************************************************
	// sends DMA destination buffer address to Master
	//*************************************************************************
    DestAddress = (uint32_t)DMASlaveDestinationAddress;
    DEBUGMSG( stdout, "Core2: Sending buffer destination address 0x%08X to master\n", DestAddress );
	master_ep = mcapi_endpoint_get(DOMAIN, NODE_CORE_0, 1, MCAPI_TIMEOUT_INFINITE,
        &mcapi_status);										// Get master endpoint
    if (MCAPI_SUCCESS != mcapi_status)
    {
#ifdef MCAPI_VERBOSE
    	mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
    	DEBUGMSG(stdout, "[ERR] mcapi_endpoint_get %s %s %d\n", status_buff, __FILE__, __LINE__);
#endif
    	return(SHARC_LINK_MCAPI_ERROR);
    }

    mcapi_msg_send(slave_ep, master_ep, &DestAddress, sizeof(DestAddress), 0, &mcapi_status);
    if (MCAPI_SUCCESS != mcapi_status)						// Send DMA transfer destination address to master
    {
#ifdef MCAPI_VERBOSE
    	mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
    	DEBUGMSG(stdout, "[ERR] mcapi_msg_send %s %s %d\n", status_buff, __FILE__, __LINE__);
#endif
    }

    DEBUGMSG( stdout, "Core2: SHARC_link connection established\n" );

	return (SHARC_LINK_SUCCESS);
}
